/*******************************************************************************/
/*                                                                             */
/*                                                                             */
/* Program: Peanuts!                                                           */
/*                                                                             */
/* Description: ``Peanuts!'' is a game of strategy and skill.                  */
/*              It is a two-player game, in which players sequentially         */
/*              eat peanuts.  When a peanut is selected, all the peanuts       */
/*              above and to the right of it in the array disappear.           */
/*              Whoever eats the rotten peanut in the lower left               */
/*              corner loses.                                                  */
/*                                                                             */
/*                                                                             */
/*******************************************************************************/
/*******************************************************************************/
/*                                                                             */
/*                                                                             */
/* Authors: Andrew Samnick and Valentin I. Spitkovsky                          */
/*                                                                             */
/* Final Project:    Computer Graphics Workshop                                */
/*                      MIT Media Laboratory                                   */
/*                       Kenneth B. Russell                                    */
/*                                                                             */
/* Date: 02/02/95                                                              */
/*                                                                             */
/* Version: 1.0                                                                */
/*                                                                             */
/*                                                                             */
/*******************************************************************************/

#include <Inventor/SoDB.h>

#include <Inventor/Xt/SoXt.h>
#include <Inventor/Xt/viewers/SoXtExaminerViewer.h>

#include <Inventor/events/SoMouseButtonEvent.h>

#include <Inventor/sensors/SoIdleSensor.h>
#include <Inventor/sensors/SoTimerSensor.h>

#include <Inventor/nodes/SoCone.h>
#include <Inventor/nodes/SoCube.h>
#include <Inventor/nodes/SoFont.h>
#include <Inventor/nodes/SoText3.h>
#include <Inventor/nodes/SoGroup.h>
#include <Inventor/nodes/SoSphere.h>
#include <Inventor/nodes/SoPerspectiveCamera.h>
#include <Inventor/nodes/SoTexture2.h>
#include <Inventor/nodes/SoRotation.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoSelection.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoEventCallback.h>
#include <Inventor/nodes/SoDirectionalLight.h>

#include <string.h>

#define MAX_M (10)
#define MAX_N (10)

#define RADIUS (5)
#define SHIFT (20)

SoXtExaminerViewer *viewer;

SoSelection *quitRoot, *gameRoot, *setupRoot;
SoSeparator *startRoot;

SoSeparator     *group[MAX_M][MAX_N];
SoSphere       *sphere[MAX_M][MAX_N];
SoMaterial   *material[MAX_M][MAX_N];
SoTransform *transform[MAX_M][MAX_N], *horShift, *allShift;

SoTimerSensor *start_rot_sens, *gover_rot_sens;

SoText3 *turn, *winner_name, *horVal, *verVal;

int M=5, N=5;

void startCB (void *, SoEventCallback *eventCB)
{
  const SoEvent *event = eventCB->getEvent ();

  if (SO_MOUSE_PRESS_EVENT (event, ANY))
    {
     eventCB->setHandled();
     viewer->setSceneGraph (setupRoot);
    }
}

void start_rotateCB (void *data, SoSensor *)
{
  SoRotation *rot = (SoRotation *)data;

  SbRotation curRot = rot->rotation.getValue ();
  curRot = SbRotation (SbVec3f (0, 1, 0), M_PI/12.0) * curRot;
  rot->rotation.setValue (curRot);
}

void make_startRoot (void)
{
  startRoot = new SoSeparator;
  startRoot->ref ();

  SoPerspectiveCamera *camera = new SoPerspectiveCamera;
  camera->position.setValue (-10.538795, -1.634, 275);
  startRoot->addChild (camera);
  startRoot->addChild (new SoDirectionalLight);

  SoEventCallback *eventCB = new SoEventCallback;
  eventCB->addEventCallback 
    (SoMouseButtonEvent::getClassTypeId (), startCB,
     viewer->getSceneManager ()->getSceneGraph ());
  startRoot->addChild (eventCB);

  SoSeparator *face = new SoSeparator;
  startRoot->addChild (face);

  SoTransform *trans0 = new SoTransform;
  trans0->translation.setValue (-75, -5, 0);
  face->addChild (trans0);

  SoTexture2 *texture = new SoTexture2;
  texture->filename.setValue ("coolpnut.rgb");
  face->addChild (texture);
 
  SoRotation *start_rot = new SoRotation;
  face->addChild (start_rot);

  start_rot_sens = 
    new SoTimerSensor (start_rotateCB, start_rot);
  start_rot_sens->schedule ();

  SoTransform *trans1 = new SoTransform;
  trans1->rotation.setValue (SbVec3f (1, 0, 0), -M_PI/6);
  face->addChild (trans1);

  SoCube *cube = new SoCube;
  cube->height.setValue (50);
  cube->width.setValue (50);
  cube->depth.setValue (50);
  face->addChild (cube);  

  SoSeparator *credits = new SoSeparator;
  startRoot->addChild (credits);

  SoTransform *trans2 = new SoTransform;
  trans2->translation.setValue (50, 0, 0);
  trans2->rotation.setValue (SbVec3f (0, 1, 0), -M_PI/6);
  credits->addChild (trans2);     

  SoSeparator *big = new SoSeparator;
  credits->addChild (big);

  SoFont *big_font = new SoFont;
  big_font->name.setValue ("Times-Roman");
  big_font->size.setValue (30);
  big->addChild (big_font);

  SoTexture2 *texture2 = new SoTexture2;
  texture2->filename.setValue ("nutture.rgb");
  big->addChild (texture2);

  SoText3 *game = new SoText3;
  game->parts = SoText3::ALL;
  game->string.setValue ("Peanuts!");
  game->justification.setValue (SoText3::CENTER);
  big->addChild (game);

  SoTransform *ntrans = new SoTransform;
  ntrans->translation.setValue (0, 50, 0);
  credits->addChild (ntrans);

  SoFont *small_font1 = new SoFont;
  small_font1->name.setValue ("Times-Roman");
  small_font1->size.setValue (10);
  credits->addChild (small_font1);

  SoText3 *names = new SoText3;
  names->parts = SoText3::ALL;
  names->string.setValue ("Drew Samnick & Val Spitkovsky");
  names->justification.setValue (SoText3::CENTER);
  credits->addChild (names);

  SoTransform *ctrans = new SoTransform;
  ctrans->translation.setValue (0, -90, 0);
  credits->addChild (ctrans);

  SoFont *small_font2 = new SoFont;
  small_font2->name.setValue ("Times-Roman");
  small_font2->size.setValue (5);
  credits->addChild (small_font2);

  SoText3 *course = new SoText3;
  course->parts = SoText3::ALL;
  course->string.set1Value (0, "Computer Graphics Workshop '96");
  course->string.set1Value (1, "MIT Media Laboratory");
  course->string.set1Value (2, "Kenneth B. Russell");
  course->string.set1Value (3, "Final Project");
  course->string.set1Value (4, "02/02/96");
  course->justification.setValue (SoText3::CENTER);
  credits->addChild (course);
}

void gover_rotateCB (void *data, SoSensor *)
{
  SoRotation *rot = (SoRotation *)data;

  SbRotation curRot = rot->rotation.getValue ();
  curRot = SbRotation (SbVec3f (0, 1, 0), M_PI/18.0) * curRot;
  rot->rotation.setValue (curRot);
}

void peanutquit_sCB (void *, SoPath *selectionPath)
{
  SoSphere *ourBall;
  char name[16];

  if (selectionPath->getTail ()->
    isOfType (SoSphere::getClassTypeId ()))
      {
       ourBall = (SoSphere*) selectionPath->getTail ();
       strcpy (name, ourBall->getName ().getString ());

       if (strcmp ("QUIT", name) == 0) exit (0);
       if (strcmp ("AGAIN", name) == 0) 
	 {
	   gover_rot_sens->unschedule();
	   viewer->setSceneGraph (setupRoot);
	 }
      }

  quitRoot->deselect (selectionPath->getTail ());
}

void make_quitRoot (void)
{
  quitRoot = new SoSelection;
  quitRoot->ref ();
  quitRoot->addSelectionCallback (peanutquit_sCB);

  SoPerspectiveCamera *camera = new SoPerspectiveCamera;
  camera->position.setValue (-1.26, -9, 250);
  quitRoot->addChild (camera);
  quitRoot->addChild (new SoDirectionalLight);

  SoSeparator *face = new SoSeparator;
  quitRoot->addChild (face);

  SoSeparator *winner = new SoSeparator;
  quitRoot->addChild (winner);

  SoFont *winner_font = new SoFont;
  winner_font->name.setValue ("Times-Roman");
  winner_font->size.setValue (20);
  winner->addChild (winner_font);

  SoTransform *winner_trans = new SoTransform;
  winner_trans->translation.setValue (0, 50, 0);
  winner_trans->rotation.setValue (SbVec3f (0, 1, 0), M_PI/6);
  winner->addChild (winner_trans);

  winner_name = new SoText3;
  winner_name->parts = SoText3::ALL;
  winner_name->justification.setValue (SoText3::CENTER);
  winner->addChild (winner_name);

  SoSeparator *gover_sep = new SoSeparator;
  quitRoot->addChild (gover_sep);

  SoFont *gover_font = new SoFont;
  gover_font->name.setValue ("Times-Roman");
  gover_font->size.setValue (40);
  gover_sep->addChild (gover_font);

  SoRotation *gover_rot = new SoRotation;
  gover_sep->addChild (gover_rot);

  gover_rot_sens = 
    new SoTimerSensor (gover_rotateCB, gover_rot);

  SoTransform *gover_trans = new SoTransform;
  gover_trans->rotation.setValue (SbVec3f (1, 0, 0), -M_PI/6);
  gover_sep->addChild (gover_trans);

  SoText3 *gover_text = new SoText3;
  gover_text->parts = SoText3::ALL;
  gover_text->string.setValue ("Game Over!");
  gover_text->justification.setValue (SoText3::CENTER);
  gover_sep->addChild (gover_text);  

  SoFont *rest_font = new SoFont;
  rest_font->name.setValue ("Times-Roman");
  rest_font->size.setValue (10);
  quitRoot->addChild (rest_font);

  SoTransform *rest_trans = new SoTransform;
  rest_trans->rotation.setValue (SbVec3f (0, 1, 0), M_PI/6);
  quitRoot->addChild (rest_trans);

  SoSeparator *pagain_sep = new SoSeparator;
  quitRoot->addChild (pagain_sep);

  SoTransform *pagain_trans = new SoTransform;
  pagain_trans->translation.setValue (0, -40, 0);
  pagain_sep->addChild (pagain_trans);

  SoText3 *pagain_text = new SoText3;
  pagain_text->parts = SoText3::ALL;
  pagain_text->string.setValue ("Would you like to play again?");
  pagain_text->justification.setValue (SoText3::CENTER);
  pagain_sep->addChild (pagain_text);

  SoTexture2 *texture = new SoTexture2;
  texture->filename.setValue ("nutture.rgb");
  quitRoot->addChild (texture);

  SoSeparator *yes_sep = new SoSeparator;
  quitRoot->addChild (yes_sep);

  SoTransform *yes_trans = new SoTransform;
  yes_trans->translation.setValue (0, -60, 0);
  yes_sep->addChild (yes_trans);
    
  SoText3 *yes_text = new SoText3;
  yes_text->parts = SoText3::ALL;
  yes_text->string.setValue ("Yes");
  yes_text->justification.setValue (SoText3::CENTER);
  yes_sep->addChild (yes_text);

  SoMaterial *yes_mat = new SoMaterial;
  yes_mat->diffuseColor.setValue (.9, .8, 0);
  yes_sep->addChild (yes_mat);

  SoTransform *yesb_trans = new SoTransform;
  yesb_trans->translation.setValue (-15, 2.5, 0);
  yes_sep->addChild (yesb_trans);

  SoSphere *yes_ball = new SoSphere;
  yes_ball->setName ("AGAIN");
  yes_ball->radius.setValue (5);
  yes_sep->addChild (yes_ball);
  
  SoSeparator *no_sep = new SoSeparator;
  quitRoot->addChild (no_sep);

  SoTransform *no_trans = new SoTransform;
  no_trans->translation.setValue (0, -75, 0);
  no_sep->addChild (no_trans);
    
  SoText3 *no_text = new SoText3;
  no_text->parts = SoText3::ALL;
  no_text->string.setValue ("No");
  no_text->justification.setValue (SoText3::CENTER);
  no_sep->addChild (no_text);

  SoMaterial *no_mat = new SoMaterial;
  no_mat->diffuseColor.setValue (0, .8, .9);
  no_sep->addChild (no_mat);

  SoTransform *nob_trans = new SoTransform;
  nob_trans->translation.setValue (-15, 2.5, 0);
  no_sep->addChild (nob_trans);

  SoSphere *no_ball = new SoSphere;
  no_ball->setName ("QUIT");
  no_ball->radius.setValue (5);
  no_sep->addChild (no_ball);  
}

void peanut_sCB (void *, SoPath *selectionPath)
{
  SoSphere *eatenPeanut;
  char name[256];
  int x, y, i, j;

  if (selectionPath->getTail ()->
    isOfType (SoSphere::getClassTypeId ()))
      {
       eatenPeanut = (SoSphere*) selectionPath->getTail ();
       strcpy (name, eatenPeanut->getName ().getString ());
       sscanf (name, "_%d_%d", &x, &y);

       for (i=x; i<M; i++)
         for (j=y; j<N; j++)
           sphere[i][j]->radius.setValue (0);

       strcpy (name, turn->getName ().getString ());

       if (x==0 && y==0)
         {
          if (strcmp (name, "Player_1_Up") == 0)
            winner_name->string.setValue ("Player 2 Wins!!!");
          else
            winner_name->string.setValue ("Player 1 Wins!!!");

	  gover_rot_sens->schedule();
          viewer->setSceneGraph (quitRoot);
         }
       else
         {
          if (strcmp (name, "Player_1_Up") == 0)
            {
             turn->string.setValue ("Player 2 Up");
             turn->setName ("Player_2_Up");
            }
          else
            {
             turn->string.setValue ("Player 1 Up");
             turn->setName ("Player_1_Up");
            }
         }
      }

  gameRoot->deselect (selectionPath->getTail ());
}

void make_gameRoot (void)
{
  int m, n;
  char name[8];

  gameRoot = new SoSelection;
  gameRoot->ref ();
  gameRoot->addSelectionCallback (peanut_sCB);

  SoPerspectiveCamera *camera = new SoPerspectiveCamera;
  camera->position.setValue (-5, 71.2, 325);
  gameRoot->addChild (camera);
  gameRoot->addChild (new SoDirectionalLight);

  allShift = new SoTransform;
  gameRoot->addChild (allShift);

  SoSeparator *who = new SoSeparator;
  gameRoot->addChild (who);

  SoTransform *PlayerTrans = new SoTransform;
  PlayerTrans->translation.setValue (0, -2*SHIFT, 0);
  PlayerTrans->rotation.setValue (SbVec3f (1, 0, 0), -M_PI/6);
  who->addChild (PlayerTrans);

  turn = new SoText3;
  turn->parts = SoText3::ALL;
  turn->justification.setValue (SoText3::CENTER);
  turn->string.setValue ("Player 1 Up");
  turn->setName ("Player_1_Up");
  who->addChild (turn);

  SoTexture2 *texture = new SoTexture2;
  texture->filename.setValue ("nutture.rgb");
  gameRoot->addChild (texture);

  horShift = new SoTransform;
  gameRoot->addChild (horShift);

  for (m=0; m<MAX_M; m++)
    for (n=0; n<MAX_N; n++)
      {
       group[m][n] = new SoSeparator;
       sphere[m][n] = new SoSphere;
       material[m][n] = new SoMaterial;
       transform[m][n] = new SoTransform;

       gameRoot->addChild (group[m][n]);
       group[m][n]->addChild (transform[m][n]);
       group[m][n]->addChild (material[m][n]);
       group[m][n]->addChild (sphere[m][n]);

       transform[m][n]->translation.setValue (m*SHIFT, n*SHIFT, 0);
       sphere[m][n]->radius.setValue (0);

       sprintf (name, "_%d_%d", m, n);
       sphere[m][n]->setName (name);
           
       material[m][n]->diffuseColor.setValue (.9, .8, 0);
      }

  material[0][0]->diffuseColor.setValue (0, .8, .9);
}

void playGame (void)
{
  int m, n;

//  for (m = 0; m < MAX_M; m++)
//    {
//      for (n = 0; n < MAX_N; n++)
//	{
//	  if ((m < M) && (n < N))
//	    sphere[m][n]->radius.setValue(RADIUS);
//	  else
//	    sphere[m][n]->radius.setValue(0);
//	}
//    }
  
  for (m=0; m<M; m++)
    for (n=0; n<N; n++)
      sphere[m][n]->radius.setValue (RADIUS);

//  for (m=M; m<MAX_M; m++)
//    for (n=N; n<MAX_N; n++)
//      sphere[m][n]->radius.setValue (0);
}

void setup_sCB (void *, SoPath *selectionPath)
{
  SoCone *ourCone;
  char name[8];

  if (selectionPath->getTail ()->
    isOfType (SoCube::getClassTypeId ()))
      {
       start_rot_sens->unschedule();
//       startRoot->unref ();
       turn->string.setValue ("Player 1 Up");
       turn->setName ("Player_1_Up");
       playGame ();
       allShift->translation.setValue (0, (MAX_N-N)*SHIFT/2, 0);
       horShift->translation.setValue (-((M-1)*SHIFT+RADIUS)/2, 0, 0);
       viewer->setSceneGraph (gameRoot);
      }
  else if (selectionPath->getTail ()->
    isOfType (SoCone::getClassTypeId ()))
      {
       ourCone = (SoCone*) selectionPath->getTail ();
       strcpy (name, ourCone->getName ().getString ());

       if (strcmp (name, "HorUp") == 0)
         {
          M++;
          if (M > MAX_M) M=MAX_M;
          horVal->string = M;
         }
       else if (strcmp (name, "HorDown") == 0)
         {
          M--;
          if (M == 1) M=2;
          horVal->string = M;
         }
       else if (strcmp (name, "VerUp") == 0)
         {
          N++;
          if (N > MAX_N) N=MAX_N;
          verVal->string = N;
         }
       else
         {
          N--;
          if (N==1) N=2;
          verVal->string = N;
         }
      }

  setupRoot->deselect (selectionPath->getTail ());
}

void make_setupRoot (void)
{
  setupRoot = new SoSelection;
  setupRoot->ref ();
  setupRoot->addSelectionCallback (setup_sCB);

  SoPerspectiveCamera *camera = new SoPerspectiveCamera;
  camera->position.setValue (-.08, -34.29, 250);
  setupRoot->addChild (camera);
  setupRoot->addChild (new SoDirectionalLight);

  SoSeparator *title = new SoSeparator;
  setupRoot->addChild (title);

  SoFont *titleFont = new SoFont;
  titleFont->name.setValue ("Times-Roman");
  titleFont->size.setValue (40);
  title->addChild (titleFont);

  SoTransform *titleTrans = new SoTransform;
  titleTrans->translation.setValue (0, 30, 0);
  titleTrans->rotation.setValue (SbVec3f (1, 0, 0), -M_PI/6);
  title->addChild (titleTrans);

  SoTexture2 *titleTexture = new SoTexture2;
  titleTexture->filename.setValue ("nutture.rgb");
  title->addChild (titleTexture);

  SoText3 *titleText = new SoText3;
  titleText->parts = SoText3::ALL;
  titleText->justification.setValue (SoText3::CENTER);
  titleText->string.setValue ("Peanuts!");
  title->addChild (titleText);

  SoSeparator *instr = new SoSeparator;
  setupRoot->addChild (instr);

  SoFont *instrFont = new SoFont;
  instrFont->name.setValue ("Times-Roman");
  instrFont->size.setValue (10);
  instr->addChild (instrFont);

  SoText3 *instrText = new SoText3;
  instrText->parts = SoText3::FRONT;
  instrText->justification.setValue (SoText3::CENTER);
  instrText->string.set1Value (0, "``Peanuts!'' is a game of strategy and skill.");
  instrText->string.set1Value (1, "It is a two-player game, in which players sequentially");
  instrText->string.set1Value (2, "eat peanuts.  When a peanut is selected, all the peanuts");
  instrText->string.set1Value (3, "above and to the right of it in the array disappear.");
  instrText->string.set1Value (4, "Whoever eats the rotten peanut in the lower left");
  instrText->string.set1Value (5, "corner, loses.  Good luck!");
  instr->addChild (instrText);

  SoSeparator *ok = new SoSeparator;
  setupRoot->addChild (ok);

  SoTransform *okTrans = new SoTransform;
  okTrans->translation.setValue (0, -100, 0);
  ok->addChild (okTrans);

  SoTexture2 *okTexture = new SoTexture2;
  okTexture->filename.setValue ("mrpnut.rgb");
  ok->addChild (okTexture);

  SoCube *okBar = new SoCube;
  okBar->width.setValue (30);
  okBar->height.setValue (45);  
  okBar->depth.setValue (2);
  ok->addChild (okBar);

  SoSeparator *under = new SoSeparator;
  setupRoot->addChild (under);

  SoFont *underFont = new SoFont;
  underFont->name.setValue ("Times-Roman");
  underFont->size.setValue (5);
  under->addChild (underFont);

  SoSeparator *left = new SoSeparator;
  under->addChild (left);

  SoTransform *leftTransform = new SoTransform;
  leftTransform->translation.setValue (-70, -110, 0);
  left->addChild (leftTransform);

  SoText3 *leftText = new SoText3;
  leftText->parts = SoText3::ALL;
  leftText->string.setValue ("Horizontal Size of the Array");
  leftText->justification.setValue (SoText3::CENTER);
  left->addChild (leftText);

  SoSeparator *right = new SoSeparator;
  under->addChild (right);

  SoTransform *rightTransform = new SoTransform;
  rightTransform->translation.setValue (70, -110, 0);
  right->addChild (rightTransform);

  SoText3 *rightText = new SoText3;
  rightText->parts = SoText3::ALL;
  rightText->string.setValue ("Vertical Size of the Array");
  rightText->justification.setValue (SoText3::CENTER);
  right->addChild (rightText);

  SoSeparator *values = new SoSeparator;
  setupRoot->addChild (values);

  SoFont *valueFont = new SoFont;
  valueFont->name.setValue ("Times-Roman");
  valueFont->size.setValue (25);
  values->addChild (valueFont);  

  SoSeparator *hor = new SoSeparator;
  values->addChild (hor);

  SoTransform *horTrans = new SoTransform;
  horTrans->translation.setValue (-65, -90, 0);
  horTrans->rotation.setValue (SbVec3f (0, 1, 0), M_PI/3);
  hor->addChild (horTrans);

  horVal = new SoText3;
  horVal->parts = SoText3::ALL;
  horVal->justification.setValue (SoText3::CENTER);
  horVal->string = M;
  hor->addChild (horVal);

  SoSeparator *ver = new SoSeparator;
  values->addChild (ver);

  SoTransform *verTrans = new SoTransform;
  verTrans->translation.setValue (65, -90, 0);
  verTrans->rotation.setValue (SbVec3f (0, 1, 0), -M_PI/3);
  ver->addChild (verTrans);

  verVal = new SoText3;
  verVal->parts = SoText3::ALL;
  verVal->justification.setValue (SoText3::CENTER);
  verVal->string = N;
  ver->addChild (verVal);

  SoSeparator *horUp = new SoSeparator;
  setupRoot->addChild (horUp);

  SoMaterial *horUpMat = new SoMaterial;
  horUpMat->diffuseColor.setValue (.7, .9, 0);
  horUp->addChild (horUpMat);

  SoTransform *horUpTrans = new SoTransform;
  horUpTrans->translation.setValue (-50, -77, 0);
  horUp->addChild (horUpTrans);

  SoCone *horUpCone = new SoCone;
  horUpCone->setName ("HorUp");
  horUpCone->bottomRadius.setValue (3);
  horUpCone->height.setValue (6);
  horUp->addChild (horUpCone);

  SoSeparator *horDown = new SoSeparator;
  setupRoot->addChild (horDown);

  SoMaterial *horDownMat = new SoMaterial;
  horDownMat->diffuseColor.setValue (.9, .7, 0);
  horDown->addChild (horDownMat);

  SoTransform *horDownTrans = new SoTransform;
  horDownTrans->translation.setValue (-50, -87, 0);
  horDownTrans->rotation.setValue (SbVec3f (1, 0, 0), M_PI);
  horDown->addChild (horDownTrans);

  SoCone *horDownCone = new SoCone;
  horDownCone->setName ("HorDown");
  horDownCone->bottomRadius.setValue (3);
  horDownCone->height.setValue (6);
  horDown->addChild (horDownCone);

  SoSeparator *verUp = new SoSeparator;
  setupRoot->addChild (verUp);

  SoMaterial *verUpMat = new SoMaterial;
  verUpMat->diffuseColor.setValue (.7, .9, 0);
  verUp->addChild (verUpMat);

  SoTransform *verUpTrans = new SoTransform;
  verUpTrans->translation.setValue (50, -77, 0);
  verUp->addChild (verUpTrans);

  SoCone *verUpCone = new SoCone;
  verUpCone->setName ("VerUp");
  verUpCone->bottomRadius.setValue (3);
  verUpCone->height.setValue (6);
  verUp->addChild (verUpCone);

  SoSeparator *verDown = new SoSeparator;
  setupRoot->addChild (verDown);

  SoMaterial *verDownMat = new SoMaterial;
  verDownMat->diffuseColor.setValue (.9, .7, 0);
  verDown->addChild (verDownMat);

  SoTransform *verDownTrans = new SoTransform;
  verDownTrans->translation.setValue (50, -87, 0);
  verDownTrans->rotation.setValue (SbVec3f (1, 0, 0), M_PI);
  verDown->addChild (verDownTrans);

  SoCone *verDownCone = new SoCone;
  verDownCone->setName ("VerDown");
  verDownCone->bottomRadius.setValue (3);
  verDownCone->height.setValue (6);
  verDown->addChild (verDownCone);
}

void main (int, char **argv)
{
  Widget window = SoXt::init (argv[0]);
  if (window == NULL) exit (1);
  viewer = new SoXtExaminerViewer 
    (window, NULL, TRUE, SoXtExaminerViewer::BUILD_NONE);

  make_startRoot ();
  make_setupRoot ();
  make_gameRoot ();
  make_quitRoot ();

  viewer->setViewing (FALSE);
  viewer->setSize (SbVec2s (1200, 900));
  viewer->setTitle ("Peanuts!");
  viewer->setSceneGraph (startRoot);
  viewer->show ();

  SoXt::show (window);
  SoXt::mainLoop ();
}
